// Package.h: interface for the Package class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PACKAGE_H__CD6DFE3C_5B70_429C_8E29_D6F967C9AC1B__INCLUDED_)
#define AFX_PACKAGE_H__CD6DFE3C_5B70_429C_8E29_D6F967C9AC1B__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <afxtempl.h> // CMap

typedef unsigned int PackSize;

//////////////////////////////////////////////////////////////////////////////////////////
// SerialToBuf
//    Basic class that can be easily converted to/from a void * pBuf.
//
class SerialToBuf : public CObject
{
public:
    DECLARE_SERIAL(SerialToBuf);
    SerialToBuf();
    virtual ~SerialToBuf();

    virtual void Serialize(CArchive & ar);

    // Use ToBuffer/FromBuffer to convert object to/from a void *
    virtual PackSize ToBuffer(BYTE *& pBuf);
    virtual PackSize FromBuffer(BYTE * pBuf, PackSize size);

};

//////////////////////////////////////////////////////////////////////////////////////////
// PackObj
//    An individual entry within the Package CMap. Each entry within a Package must be 
//    serializable.
//
class PackObj : public SerialToBuf
{
public:
    enum StoreType {P_OBJECT, P_STRING, P_64};
    union ValueBuf {
        double                  d_fp64;
        float                   d_fp32;
        unsigned __int64        d_uint64;
        __int64                 d_int64;
        unsigned int            d_uint32;
        int                     d_int32;
        unsigned short          d_uint16;
        short                   d_int16;
        BYTE                    d_byte;
        char                    d_char;
        BYTE pBuf[8];
    };
public:
    DECLARE_SERIAL(PackObj);
    PackObj();
    virtual ~PackObj();
    PackObj & operator=(const PackObj & obj);
    PackSize CopyBuf(BYTE *& pBuffer) const;
    virtual void Serialize(CArchive & ar);    
    bool Get(CString & objName, CObject * pObj);
    bool Set(const CString & objName, CObject * pObj);
    bool Get(CString & objName, CString & str);
    bool Set(const CString & objName, const CString & str);
    bool Get(CString & objName, ValueBuf & vb);
    bool Set(const CString & objName, const ValueBuf & vb);
    
    CString Name()  { return name; }
    StoreType Type() { return sType; }
private:
    CString name;
    StoreType sType;
    BYTE * pBuf;
    PackSize bufSize;
};

//////////////////////////////////////////////////////////////////////////////////////////
// Package
//    Used to package up many objects into one object that can be easily Serialized or
//    converted to a BYTE * pBuf.
//
class Package : public SerialToBuf
{
public:
    DECLARE_SERIAL(Package);
    Package();
    virtual ~Package();
    Package & operator=(const Package & pack);
    void Copy(const Package & pack);
    virtual void Serialize(CArchive & ar);
    void freeMemory();

    // Get/Set CObject
    bool Get(const CString & objName, CObject * pObj) const;
    bool Set(const CString & objName, CObject * pObj);

    // Get/Set String
    bool Get(const CString & objName, CString & str) const;
    bool Set(const CString & objName, const CString & str);

    // Get/Set u64bit number
    bool Get(const CString & objName, unsigned __int64 & n64) const;
    bool Set(const CString & objName, unsigned __int64 n64);

    // Get/Set 64bit number
    bool Get(const CString & objName, __int64 & n64) const;
    bool Set(const CString & objName, __int64 n64);

    // Get/Set double
    bool Get(const CString & objName, double & dnum) const;
    bool Set(const CString & objName, double dnum);

    // Get/Set u32bit number
    bool Get(const CString & objName, UINT & n32) const;
    bool Set(const CString & objName, UINT n32);

    // Get/Set 32bit number
    bool Get(const CString & objName, int & n32) const;
    bool Set(const CString & objName, int n32);

    // Get/Set 32bit number
    bool Get(const CString & objName, float & fnum) const;
    bool Set(const CString & objName, float fnum);

    // Get/Set u16bit number
    bool Get(const CString & objName, unsigned short & n16) const;
    bool Set(const CString & objName, unsigned short n16);

    // Get/Set 16bit number
    bool Get(const CString & objName, short & n16) const;
    bool Set(const CString & objName, short n16);

    // Get/Set byte
    bool Get(const CString & objName, BYTE & n8) const;
    bool Set(const CString & objName, BYTE n8);

    // Get/Set char
    bool Get(const CString & objName, char & n8) const;
    bool Set(const CString & objName, char n8);

    CMap<CString, const char *, PackObj *, PackObj *> map;
};

#endif // !defined(AFX_PACKAGE_H__CD6DFE3C_5B70_429C_8E29_D6F967C9AC1B__INCLUDED_)
